==> 1.0.2.tar.gz: android/src/main/kotlin/com/anagog/anagog_flutter/AnagogFlutterPlugin.kt <==
package com.anagog.anagog_flutter

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.anagog.anagog_flutter.jedai_sdk.JemaWrapper
import com.anagog.anagog_flutter.model.Micromoment
import com.google.gson.Gson

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler

/** AnagogFlutterPlugin */
class  AnagogFlutterPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel: MethodChannel
  private var  eventsChannel : EventChannel? = null
  private var context: Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    Log.i(TAG, "onAttachedToEngine: ")
    setMethodChannel(flutterPluginBinding)
    setEventChannel(flutterPluginBinding)
    context = flutterPluginBinding.applicationContext
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    Log.i(TAG, "onDetachedFromEngine: ")
    channel.setMethodCallHandler(null)
    context = null
    JemaWrapper.closeEventSync()
    eventsChannel = null
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
    when (call.method) {
      "simulateCampaign" -> {
        Log.i(TAG, "simulateCampaign: ")
        val campaignIdentifier = call.argument<String>("campaign_identifier")
        JemaWrapper.simulateCampaign(campaignIdentifier)
        result.success(null)
      }
      "syncCampaigns" -> {
        Log.i(TAG, "syncCampaigns: ")
        JemaWrapper.syncCampaigns()
        result.success(null)
      }
      "getUserDefinedStat" -> {
        Log.i(TAG, "getUserDefinedStat:")
        JemaWrapper.getUserDefinedStats(result)
      }
      "setUserDefinedString" -> {
        Log.i(TAG, "setUserDefinedString: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedString(key, value)
        }
      }
      "getUserDefinedString" -> {
        Log.i(TAG, "getUserDefinedString: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedString(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "setUserDefinedInteger" -> {
        Log.i(TAG, "setUserDefinedInteger: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedInteger(key, value.toInt())
        }
      }
      "getUserDefinedInteger" -> {
        Log.i(TAG, "getUserDefinedInteger: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedInteger(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "setUserDefinedDecimal" -> {
        Log.i(TAG, "setUserDefinedDecimal: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedDecimal(key, value.toDouble())
        }
      }
      "getUserDefinedDecimal" -> {
        Log.i(TAG, "getUserDefinedInteger: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedDecimal(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "fireMicromoments" -> {
        Log.i(TAG, "invokeMethod: fireMicromoments: arguments: ${call.arguments}")
        val micromoment = Gson().fromJson(call.arguments.toString(), Micromoment::class.java)
        JemaWrapper.fireMicromoment(micromoment)
        result.success(null)
      }
      "share" -> {
        Log.i(TAG, "invokeMethod: share: ")
        context?.let{
          JemaWrapper.share(it)
        }
      }
      "getVersion" -> {
        Log.i(TAG, "invokeMethod: getVersion: ")
        JemaWrapper.getVersion(result)
      }
      "requestMicrosegmentsSnapshotReport" -> {
        Log.i(TAG, "invokeMethod: requestMicrosegmentsSnapshotReport: ")
        JemaWrapper.requestMicrosegmentsSnapshotReport()
        result.success(null)
      }

      "forceScheduleReports" -> {
        Log.i(TAG, "invokeMethod: forceScheduleReports: ")
        JemaWrapper.forceScheduleReports()
        result.success(null)
      }
      else -> result.notImplemented()
    }
  }



  private fun setMethodChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, METHOD_CHANNEL)
    channel.setMethodCallHandler(this)
  }

  private fun setEventChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    eventsChannel = EventChannel(flutterPluginBinding.binaryMessenger, EVENT_CHANNEL)
    eventsChannel?.setStreamHandler(object : EventChannel.StreamHandler{
      override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        Log.i(TAG, "setEventChannel: onListen")
        events?.let{
          JemaWrapper.setEventSink(it)
        }
      }

      override fun onCancel(arguments: Any?) {
        JemaWrapper.closeEventSync()
        Log.i(TAG, "setEventChannel: onCancel")
      }
    })
  }

  companion object {
    const val TAG = "XXX: JemaAndroidPlugin"
    const val METHOD_CHANNEL = "com.anagog.flutter/method_channel"
    const val EVENT_CHANNEL = "com.anagog.flutter/method_channel/events"
  }
}

==> 1.1.0.tar.gz: android/src/main/kotlin/com/anagog/anagog_flutter/AnagogFlutterPlugin.kt <==
package com.anagog.anagog_flutter

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.anagog.anagog_flutter.jedai_sdk.JemaWrapper
import com.anagog.anagog_flutter.model.Micromoment
import com.google.gson.Gson

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler

/** AnagogFlutterPlugin */
class  AnagogFlutterPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel: MethodChannel
  private var  eventsChannel : EventChannel? = null
  private var context: Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    Log.i(TAG, "onAttachedToEngine: ")
    setMethodChannel(flutterPluginBinding)
    setEventChannel(flutterPluginBinding)
    context = flutterPluginBinding.applicationContext
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    Log.i(TAG, "onDetachedFromEngine: ")
    channel.setMethodCallHandler(null)
    context = null
    JemaWrapper.closeEventSync()
    eventsChannel = null
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
    when (call.method) {
      "simulateCampaign" -> {
        Log.i(TAG, "simulateCampaign: ")
        val campaignIdentifier = call.argument<String>("campaign_identifier")
        JemaWrapper.simulateCampaign(campaignIdentifier)
        result.success(null)
      }
      "syncCampaigns" -> {
        Log.i(TAG, "syncCampaigns: ")
        JemaWrapper.syncCampaigns()
        result.success(null)
      }
      "getUserDefinedStat" -> {
        Log.i(TAG, "getUserDefinedStat:")
        JemaWrapper.getUserDefinedStats(result)
      }
      "setUserDefinedString" -> {
        Log.i(TAG, "setUserDefinedString: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedString(key, value)
        }
      }
      "getUserDefinedString" -> {
        Log.i(TAG, "getUserDefinedString: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedString(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "setUserDefinedInteger" -> {
        Log.i(TAG, "setUserDefinedInteger: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedInteger(key, value.toInt())
        }
      }
      "getUserDefinedInteger" -> {
        Log.i(TAG, "getUserDefinedInteger: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedInteger(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "setUserDefinedDecimal" -> {
        Log.i(TAG, "setUserDefinedDecimal: arguments: ${call.arguments}")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val key = arguments["key"].toString()
          val value = arguments["value"].toString()
          JemaWrapper.setUserDefinedDecimal(key, value.toDouble())
        }
      }
      "getUserDefinedDecimal" -> {
        Log.i(TAG, "getUserDefinedInteger: arguments: ${call.arguments}")
        val key = call.arguments as String?
        key?.let {
          JemaWrapper.getUserDefinedDecimal(it, result)
        } ?: kotlin.run {
          result.error("0",   "invalid parameter",   null)
        }
      }
      "fireMicromoments" -> {
        Log.i(TAG, "invokeMethod: fireMicromoments: arguments: ${call.arguments}")
        val micromoment = Gson().fromJson(call.arguments.toString(), Micromoment::class.java)
        JemaWrapper.fireMicromoment(micromoment)
        result.success(null)
      }
      "share" -> {
        Log.i(TAG, "invokeMethod: share: ")
        context?.let{
          JemaWrapper.share(it, result)
        }
      }
      "getVersion" -> {
        Log.i(TAG, "invokeMethod: getVersion: ")
        JemaWrapper.getVersion(result)
      }
      "requestMicrosegmentsSnapshotReport" -> {
        Log.i(TAG, "invokeMethod: requestMicrosegmentsSnapshotReport: ")
        JemaWrapper.requestMicrosegmentsSnapshotReport()
        result.success(null)
      }
      "forceScheduleReports" -> {
        Log.i(TAG, "invokeMethod: forceScheduleReports: ")
        JemaWrapper.forceScheduleReports()
        result.success(null)
      }
      "setUserInteractionEvent" -> {
        Log.i(TAG, "invokeMethod: forceScheduleReports: ")
        val arguments = call.arguments
        if(arguments is Map<*, *>) {
          val campaignIdentifier = arguments["campaign_identifier"].toString()
          val activityType = arguments["activity_type"].toString()
          JemaWrapper.setUserInteractionEvent(campaignIdentifier, activityType)
          result.success(null)
        } else {
          result.error("", "Invalid arguments for userInteractionEvent", "Make sure you provide map like {'campaign_identifier': '..', 'activity_type':'..'}")
        }
      }
      else -> result.notImplemented()
    }
  }



  private fun setMethodChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, METHOD_CHANNEL)
    channel.setMethodCallHandler(this)
  }

  private fun setEventChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    eventsChannel = EventChannel(flutterPluginBinding.binaryMessenger, EVENT_CHANNEL)
    eventsChannel?.setStreamHandler(object : EventChannel.StreamHandler{
      override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        Log.i(TAG, "setEventChannel: onListen")
        events?.let{
          JemaWrapper.setEventSink(it)
        }
      }

      override fun onCancel(arguments: Any?) {
        JemaWrapper.closeEventSync()
        Log.i(TAG, "setEventChannel: onCancel")
      }
    })
  }

  companion object {
    const val TAG = "XXX: JemaAndroidPlugin"
    const val METHOD_CHANNEL = "com.anagog.flutter/method_channel"
    const val EVENT_CHANNEL = "com.anagog.flutter/method_channel/events"
  }
}
